from fastapi import FastAPI
import uvicorn
import fire

from glockr.manager import GResourceManager
from glockr import config

# --- API below ---

# TODO API for updating resource?

app = FastAPI()


@app.get(config.ROUTER['heartbeat'])
def heartbeat():
    return {"hello": "from glockr server"}


@app.get(config.ROUTER['show_all'])
def show_all():
    return GResourceManager.show_all()


@app.post(config.ROUTER['add'])
def add(name: str, label: str = None):
    return GResourceManager.add(name, label).to_dict()


@app.post(config.ROUTER['remove'])
def remove(name: str):
    return GResourceManager.remove(name).to_dict()


@app.post(config.ROUTER['acquire'])
def acquire(name: str = None, label: str = None):
    """ acquire resource """
    if name:
        return GResourceManager.acquire_res(name).to_dict()
    if label:
        return GResourceManager.acquire_label(label).to_dict()
    return {
        'error': 'at least fill name or label'
    }


@app.post(config.ROUTER['release'])
def release(name: str = None, label: str = None):
    """ release resource """
    if name:
        return GResourceManager.release_res(name).to_dict()
    if label:
        return GResourceManager.release_label(label).to_dict()
    return {
        'error': 'at least fill name or label'
    }


def start_server(port: int = None):
    if not port:
        port = int(config.PORT)

    uvicorn.run(
        app,
        host="0.0.0.0",
        port=port,
        log_level="info"
    )


def main():
    fire.Fire({
        'start': start_server,
    })


if __name__ == '__main__':
    main()



import os


PORT_ENV_NAME = 'GLOCKR_PORT'
if PORT_ENV_NAME in os.environ:
    PORT = os.environ[PORT_ENV_NAME]
else:
    PORT = '29410'

CHARSET = 'utf-8'

ROUTER = {
    'heartbeat': '/',
    'add': '/res/add',
    'remove': '/res/remove',
    'show_all': '/res',
    'acquire': '/res/acquire',
    'release': '/res/release',
}



import unittest
import subprocess
import requests
import time
import json

from glockr.config import PORT
from glockr.client import GClient

BASE_URL = 'http://127.0.0.1:{}'.format(PORT)


class TestClient(unittest.TestCase):
    _server_process = None

    @staticmethod
    def get_current_dict() -> dict:
        target_url = BASE_URL + '/res'
        resp = requests.get(target_url)
        resp_dict = json.loads(resp.text)
        return resp_dict

    @classmethod
    def setUpClass(cls):
        super(TestClient, cls).setUpClass()
        cls._server_process = subprocess.Popen('python3 -m glockr.server start', shell=True)
        time.sleep(3)

    def test_1_add(self):
        name_list = list(range(0, 30))
        for label in range(5):
            for _ in range(5):
                GClient.add(str(name_list.pop()), str(label))

    def test_2_show_all(self):
        current_dict = GClient.show_all()
        assert len(current_dict) == 25

    def test_3_acquire_name(self):
        # check 15-19
        for each_name in range(15, 20):
            each_name = str(each_name)
            GClient.acquire('name', each_name)

            # try to acquire it again
            resp = GClient.acquire('name', each_name)
            assert not resp['result']

            # try to acquire by label
            resp = GClient.acquire('label', str(2))
            assert not resp['result']

            # release
            resp = GClient.release('name', each_name)
            assert resp['result']

            # check its status
            resp = GClient.show_all()
            assert resp[each_name]['status'] == 'FREE'

    @classmethod
    def tearDownClass(cls):
        super(TestClient, cls).tearDownClass()
        cls._server_process.kill()


if __name__ == '__main__':
    unittest.main()



import requests
import fire
import json
import requests.exceptions

from glockr import config

# TODO API for waiting until release

_PORT = config.PORT
_BASE_URL = 'http://127.0.0.1:{}'.format(_PORT)

_URL_DICT = {
    each_name: _BASE_URL + each_router
    for each_name, each_router in config.ROUTER.items()
}


class GClient(object):
    @classmethod
    def heartbeat(cls) -> bool:
        target_url = _URL_DICT['heartbeat']
        try:
            resp = requests.get(target_url)
        except requests.exceptions.ConnectionError:
            return False
        return resp.ok

    @classmethod
    def add(cls, name: str, label: str = None) -> dict:
        target_url = _URL_DICT['add'] + '?name={}'.format(name)
        if label:
            target_url += '&label={}'.format(label)
        resp = requests.post(target_url)
        return resp.json()

    @classmethod
    def remove(cls, name: str) -> dict:
        target_url = _URL_DICT['remove'] + '?name={}'.format(name)
        resp = requests.post(target_url)
        return resp.json()

    @classmethod
    def show_all(cls) -> dict:
        target_url = _URL_DICT['show_all']
        resp = requests.get(target_url)
        return resp.json()

    @classmethod
    def download(cls, result_file_path: str) -> dict:
        target_url = _URL_DICT['show_all']
        resp = requests.get(target_url)

        with open(result_file_path, 'w+', encoding=config.CHARSET) as f:
            f.write(resp.text)

        return resp.json()

    @classmethod
    def upload(cls, result_file_path: str) -> list:
        result_list = list()
        with open(result_file_path, encoding=config.CHARSET) as f:
            data_dict = json.loads(f.read())
        for each_data in data_dict.values():
            each_result = cls.add(each_data['name'], each_data['label'])
            result_list.append(each_result)
        return result_list

    @classmethod
    def acquire(cls, acquire_type: str, content: str) -> dict:
        target_url = _URL_DICT['acquire'] + '?{}={}'.format(acquire_type, content)
        resp = requests.post(target_url)
        return resp.json()

    @classmethod
    def release(cls, release_type: str, content: str) -> dict:
        target_url = _URL_DICT['release'] + '?{}={}'.format(release_type, content)
        resp = requests.post(target_url)
        return resp.json()


def output2console(output_content: (list, dict)):
    print(json.dumps(output_content))


class GClient4CLI(object):
    @classmethod
    def heartbeat(cls) -> bool:
        return GClient.heartbeat()

    @classmethod
    def acquire_name(cls, name: str):
        output2console(GClient.acquire('name', name))

    @classmethod
    def acquire_label(cls, label: str):
        output2console(GClient.acquire('label', label))

    @classmethod
    def release_name(cls, name: str):
        output2console(GClient.release('name', name))

    @classmethod
    def release_label(cls, label: str):
        output2console(GClient.release('label', label))

    @classmethod
    def add(cls, name: str, label: str = None):
        output2console(GClient.add(name, label))

    @classmethod
    def remove(cls, name: str):
        output2console(GClient.remove(name))

    @classmethod
    def show_all(cls):
        output2console(GClient.show_all())

    @classmethod
    def upload(cls, result_file_path: str):
        output2console(GClient.upload(result_file_path))

    @classmethod
    def download(cls, result_file_path: str):
        output2console(GClient.download(result_file_path))


def main():
    fire.Fire(GClient4CLI)


if __name__ == '__main__':
    main()
